<?php
if ( ! class_exists( 'IPRev_Viewer' ))
{
  /**
   * Viewer class
   *
   * @package wp-review-me
   * @author InstaProduct
   * @link http://insta-product.com/
   */

  class IPRev_Viewer
  {
    //{{ dashboard

    /**
     * Manage page viewer
     *
     * @return void
     */
    public function dashboard()
    {
      //Render view
      $this->pageId = IPREV_PAGENAME;
      $this->view( 'manage/dashboard' );
    }

    //}}
    //{{ page

    /**
     * Manage review page viewer
     *
     * @return void
     */
    public function page()
    {
      //Render view
      $this->pageId = IPREV_PAGENAME . '-page';
      $this->view( 'page/dashboard' );
    }


    //}}
    //{{ createPage

    /**
     * Add new review page viewer
     *
     * @return void
     */
    public function createPage()
    {
      $this->managePage();
    }

    //}}
    //{{ editPage

    /**
     * Edit review page viewer
     *
     * @return void
     */
    public function editPage()
    {
      if ( empty( $_GET['id'] )) {
        wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPREV_SLUG ) );
      }
      $id = absint( $_GET['id'] );
      if ( ! $page = get_post( $id ) ) {
        wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPREV_SLUG ) );
      }
      $this->managePage( $page );
    }

    //}}
    //{{ managePage

    /**
     * Manage review page viewer
     *
     * @param object $page
     * @return void
     */
    public function managePage( $page = false )
    {
      //
      //Including script
      wp_enqueue_media();
      wp_enqueue_script('wp-color-picker');
      wp_enqueue_style('wp-color-picker');

      $itemId = 0;
      if ( $page ) {
        $itemId = $page->ID;
      }
      $setting    = new IPRev_Setting( $itemId, '__iprev-page-setting' );
      $formAction = admin_url( 'admin-ajax.php' ) . '?doajax=1&iprev=1&action=update_page';
      if ( $page )
      {
        $pageTitle   = __( 'Edit Review Page', IPREV_SLUG );
        $submitLabel = __( 'Save Change', IPREV_SLUG );
        $submitId    = 'iprev-edit-page';
        $formId      = 'iprev-form-page-update';
        $submitIcon  = 'floppy-o';
        $pageId      = IPREV_PAGENAME . '-page';

      } else
      {
        $pageTitle   = __( 'Add Review Page', IPREV_SLUG );
        $submitLabel = __( 'Add Review Page', IPREV_SLUG );
        $submitId    = 'iprev-create-page';
        $formId      = 'iprev-form-page-create';
        $submitIcon  = 'plus-circle';
        $pageId      = IPREV_PAGENAME . '-page';

        $this->title = __( 'Add Review Page', IPREV_SLUG );
        $this->info  = __( 'Create new review page', IPREV_SLUG );
        $this->icon  = 'plus-alt';
      }
      $this->pageId      = $pageId;
      $this->pageTitle   = $pageTitle;
      $this->submitLabel = $submitLabel;
      $this->submitId    = $submitId;
      $this->formId      = $formId;
      $this->submitIcon  = $submitIcon;
      $this->setting     = $setting;
      $this->page        = $page;
      $this->item        = $page;
      $this->itemId      = $itemId;
      $this->formAction  = $formAction;

      //
      //Get Unapproved review
      //
      $count = 0;
      if ( $counts = wp_count_comments( $itemId )) {
        $count = $counts->moderated;
      }
      $this->moderatedCount = $count;

      $tab = 'iprev-section-general';
      if ( ! empty( $_REQUEST['tab'] )) {
        $tab = $_REQUEST['tab'];
      }
      $this->tab = $tab;

      //
      //Permalink root
      $home = get_home_url();
      $path = get_option( '__iprev_review_page_base_path', 'review' );
      $root = $home . '/' . $path . '/';
      $this->permalink_root = $root;

      //Render view
      $this->view( 'page/main' );
    }

    //}}
    //{{ style

    /**
     * Manage style page viewer
     *
     * @return void
     */
    public function style()
    {
      //Render view
      $this->pageId = IPREV_PAGENAME . '-style';
      $this->view( 'style/dashboard' );
    }

    //}}
    //{{ createStyle

    /**
     * Add new style page viewer
     *
     * @return void
     */
    public function createStyle()
    {
      $this->manageStyle();
    }

    //}}
    //{{ editStyle

    /**
     * Edit style page viewer
     *
     * @return void
     */
    public function editStyle()
    {
      if ( empty( $_GET['id'] )) {
        wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPREV_SLUG ) );
      }
      $id = absint( $_GET['id'] );
      if ( ! $style = get_post( $id ) ) {
        wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPREV_SLUG ) );
      }
      $this->manageStyle( $style );
    }

    //}}
    //{{ manageStyle

    /**
     * Manage style page viewer
     *
     * @param object $style
     * @return void
     */
    public function manageStyle( $style = false )
    {
      //
      //Including script
      wp_enqueue_media();
      wp_enqueue_script('wp-color-picker');
      wp_enqueue_style('wp-color-picker');

      $styleId = 0;
      if ( $style ) {
        $styleId = $style->ID;
      }
      $setting    = new IPRev_Setting( $styleId, '__iprev-style-setting' );
      $formAction = admin_url( 'admin-ajax.php' ) . '?doajax=1&iprev=1&action=update_style';
      if ( $style )
      {
        $pageTitle   = __( 'Edit Style', IPREV_SLUG );
        $submitLabel = __( 'Save Change', IPREV_SLUG );
        $submitId    = 'iprev-edit-style';
        $formId      = 'iprev-form-style-update';
        $submitIcon  = 'floppy-o';
        $pageId      = IPREV_PAGENAME . '-style';

      } else
      {
        $pageTitle   = __( 'Add Review Style', IPREV_SLUG );
        $submitLabel = __( 'Add Review Style', IPREV_SLUG );
        $submitId    = 'iprev-add-style';
        $formId      = 'iprev-form-style-create';
        $submitIcon  = 'plus-circle';
        $pageId      = IPREV_PAGENAME . '-style';

        $this->title = __( 'Add Review Style', IPREV_SLUG );
        $this->info  = __( 'Create new review style', IPREV_SLUG );
        $this->icon  = 'plus-alt';
      }
      $this->pageId      = $pageId;
      $this->pageTitle   = $pageTitle;
      $this->submitLabel = $submitLabel;
      $this->submitId    = $submitId;
      $this->formId      = $formId;
      $this->submitIcon  = $submitIcon;
      $this->setting     = $setting;
      $this->style       = $style;
      $this->styleId     = $styleId;
      $this->formAction  = $formAction;

      $tab = 'iprev-section-general';
      if ( ! empty( $_REQUEST['tab'] )) {
        $tab = $_REQUEST['tab'];
      }
      $this->tab = $tab;

      //Render view
      $this->view( 'style/main' );
    }

    //}}
    //{{ setting

    /**
     * Setting page viewer
     *
     * @return void
     */
    public function setting()
    {
        $this->styles = $this->_getStyles();
        $this->pages  = $this->_getPages();
        $this->pageId = IPREV_PAGENAME . '-setting';
        //Render view
        $this->view( 'manage/setting' );
    }

    //}}
    //{{ shortcodeBackend

    /**
     * Shortcode Backend viewer
     *
     * @return void
     */
    public function shortcodeBackend()
    {
      $this->styles = $this->_getStyles();
      $this->pages  = $this->_getPages();
      //Render view
      $this->view( 'manage/shortcode' );
    }

    //}}
    //{{ metabox

    /**
     * Metabox options viewer
     *
     * @return void
     */
    public function metabox( $post )
    {
        $this->post   = $post;
        $this->styles = $this->_getStyles();
        $this->pages  = $this->_getPages();
        //Render view
        $this->view( 'manage/metabox' );
    }

    //}}
    //{{ metaboxComment

    /**
     * Metabox comment options viewer
     *
     * @return void
     */
    public function metaboxComment( $comment )
    {
      if ( $comment->comment_type == 'iprev_item' )
      {
        $this->comment = $comment;

        //Render view
        $this->view( 'manage/metabox-comment' );
      }
    }

    //}}
    //{{ widgetBackend

    /**
     * Widget Backend viewer
     *
     * @param array $instance
     * @param object $wp_widget
     * @return void
     */
    public function widgetBackend( $instance, $wp_widget )
    {
      $this->styles = $this->_getStyles();
      $this->pages  = $this->_getPages();
      $this->instance  = $instance;
      $this->wp_widget = $wp_widget;
      //Render view
      $this->view( 'manage/widget' );
    }

    //}}
    //{{ reviewPage

    /**
     * Widget frontend viewer
     *
     * @return void
     */
    public function reviewPage()
    {
      global $wp_query, $user_ID;
      if ( ( isset( $wp_query->query['iprev'] ) && isset( $wp_query->query['review_name'] ) ) ||
           isset( $wp_query->query['review_id'] ) )
      {
        $page = $wp_query->post;

        if ( $page->post_status == 'publish' )
        {
          $setting = new IPRev_Setting( $page->ID, '__iprev-page-setting' );

          //Check restricted
          if ( empty( $user_ID ) && $setting->getSetting( 'restricted', 'false' ) == 'true' )
          {
              $redirect  = $this->permalink;
              $login_url = wp_login_url( $redirect );
              wp_redirect( $login_url );
              exit;
          }

          $this->query     = $wp_query;
          $this->setting   = $setting;
          $this->page      = $page;
          $this->cssurl    = IPREV_BASE_URL . 'assets/css/';
          $this->jsurl     = IPREV_BASE_URL . 'assets/js/';
          $this->imgurl    = IPREV_BASE_URL . 'assets/img/';
          $this->permalink = $this->_getPermalink( $page );

          $title = $this->setting->getSetting( 'seo_meta_title' );
          $title = trim( $title );
          if ( empty( $title )) {
              $title = $this->page->post_title;
          }
          $this->pageTitle = stripslashes( $title );

          //Render view
          $this->view( 'front/page/main' );
          exit;
        }
        else
        {
          $query->query_vars['post_type'] = 'iprev-item-nothing';
        }
      }
    }

    //}}
    //{{ shortcode

    /**
     * Shortcode frontend viewer
     *
     * @param array  $atts
     * @param string $content
     * @return void
     */
    public function shortcode( $atts, $content = '' )
    {
      global $user_ID;

      if ( empty( $atts['review_page'] ) || empty( $atts['style'] ) ) {
          return '';
      }
      //
      //Get content
      $pageId = absint( $atts['review_page'] );
      if ( ! $page = get_post( $pageId )) {
          return '';
      }
      $styleId = absint( $atts['style'] );
      if ( ! $style = get_post( $styleId )) {
          return '';
      }

      $this->reviewPage   = $page;
      $this->style        = $style;
      $this->pageSetting  = new IPRev_Setting( $page->ID, '__iprev-page-setting' );
      $this->styleSetting = new IPRev_Setting( $style->ID, '__iprev-style-setting' );
      $this->reviews      = $this->_getReviews( $page->ID, $this->styleSetting );
      $this->reviewCounts = wp_count_comments( $page->ID );
      $this->pageRating   = $this->_getPageRating( $page->ID );

      //Render view
      ob_start();
      $this->view( 'front/widget/post' );
      $output = ob_get_contents();
      ob_end_clean();

      return $output;
    }

    //}}
    //{{ comboBar

    /**
     * Combo bar viewer
     *
     * @return void
     */
    public function comboBar()
    {
      $reviewPage  = $this->_getReviewPage();
      $widgetStyle = $this->_getWidgetStyle();

      if ( ! empty( $reviewPage['page'] ) && ! empty( $widgetStyle['style'] ))
      {
        $page  = $reviewPage['page'];
        $type  = $reviewPage['type'];
        $style = $widgetStyle['style'];

        $this->reviewPage   = $page;
        $this->style        = $style;
        $this->pageSetting  = new IPRev_Setting( $page->ID, '__iprev-page-setting' );
        $this->styleSetting = new IPRev_Setting( $style->ID, '__iprev-style-setting' );
        $this->reviews      = $this->_getReviews( $page->ID, $this->styleSetting );
        $this->reviewCounts = wp_count_comments( $page->ID );
        $this->pageRating   = $this->_getPageRating( $page->ID );

        //
        //Include view
        $this->view( 'front/widget/bar' );
      }
    }

    //}}
    //{{ widgetFrontend

    /**
     * Widget frontend viewer
     *
     * @param array $atts
     * @param array $widgetArgs
     * @return void
     */
    public function widgetFrontend( $atts, $widgetArgs = array() )
    {
      global $user_ID;

      if ( empty( $atts['review_page'] ) || empty( $atts['style'] ) ) {
        return '';
      }

      //
      //Get content
      $pageId = absint( $atts['review_page'] );
      if ( ! $page = get_post( $pageId )) {
          return '';
      }
      $styleId = absint( $atts['style'] );
      if ( ! $style = get_post( $styleId )) {
          return '';
      }

      $this->reviewPage   = $page;
      $this->style        = $style;
      $this->pageSetting  = new IPRev_Setting( $page->ID, '__iprev-page-setting' );
      $this->styleSetting = new IPRev_Setting( $style->ID, '__iprev-style-setting' );
      $this->reviews      = $this->_getReviews( $page->ID, $this->styleSetting );
      $this->reviewCounts = wp_count_comments( $page->ID );
      $this->pageRating   = $this->_getPageRating( $page->ID );

      //
      //Include view
      $this->view( 'front/widget/sidebar' );
    }

    //}}
    //{{ _getReviewPage

    /**
     * Get review page
     *
     * @return void
     */
    protected function _getReviewPage()
    {
      global $post;

      $pageId  = 0;
      $page    = false;
      $type    = 'general';
      $general = get_option( '__iprev_general_review_page' );
      if ( ! empty( $general )) {
        $pageId = absint( $general );
        $type   = 'general';
      } else
      {
        if ( ( is_page() || is_single() ) && ! empty( $post )) {
          $pageId = get_post_meta( $post->ID, '__iprev_page_review_page', true );
          $type   = 'page';
        } elseif ( is_home() ) {
          $pageId = get_option( '__iprev_home_review_page' );
          $type   = 'home';
        }
      }
      if ( ! empty( $pageId )) {
        $page = get_post( $pageId );
      }
      return array(
        'page' => $page,
        'type' => $type
      );
    }

    //}}
    //{{ _getWidgetStyle

    /**
     * Get widget style
     *
     * @return void
     */
    protected function _getWidgetStyle()
    {
      global $post;

      $styleId = 0;
      $style   = false;
      $type    = 'general';
      $general = get_option( '__iprev_general_style' );
      if ( ! empty( $general )) {
        $styleId = absint( $general );
        $type    = 'general';
      } else
      {
        if ( ( is_page() || is_single() ) && ! empty( $post )) {
          $styleId = get_post_meta( $post->ID, '__iprev_page_style', true );
          $type    = 'page';
        } elseif ( is_home() ) {
          $styleId = get_option( '__iprev_home_style' );
          $type    = 'home';
        }
      }
      if ( ! empty( $styleId )) {
        $style = get_post( $styleId );
      }
      return array(
        'style' => $style,
        'type'  => $type
      );
    }

    //}}
    //{{ view

    /**
     * Get admin view file
     *
     * @param string $filename
     * @return void
     */
    public function view( $filename )
    {
      $path  = IPREV_VIEW;
      $path .= strtolower( $filename ) . '.php';
      if ( file_exists( $path )) {
        include( $path );
      }
    }

    //}}
    //{{ _getPermalink

    /**
     * Get link view
     *
     * @param  object $item
     * @return string
     */
    protected function _getPermalink( $item )
    {
      $home = get_home_url();
      $base = get_option( '__iprev_review_page_base_path', 'review' );

      if ( get_option( 'permalink_structure' ) )  {
        $link = $home . "/$base/" . $item->post_name;
      } else {
        $link = $home . '/?review_id=' . $item->ID . '&iprev=1';
      }

      return $link;
    }

    //}}
    //{{ _getReviews

    /**
     * Get reviews list
     *
     * @return array
     */
    protected function _getReviews( $pageId, $style )
    {
      $count   = $style->getSetting( 'count', 10 );
      $orderby = $style->getSetting( 'orderby', 'newest' );
      $args    = array(
          'type'    => 'iprev_item',
          'status'  => 'approve',
          'post_id' => $pageId,
          'number'  => $count,
          'offset'  => 0
      );
      if ( $orderby == 'oldest' ) {
        $args['orderby'] = 'comment_date_gmt';
        $args['order']   = 'ASC';
      }
      elseif ( $orderby == 'rating' ) {
        $args['orderby'] = 'comment_karma';
        $args['order']   = 'DESC';
      }
      else {
        $args['orderby'] = 'comment_date_gmt';
        $args['order']   = 'DESC';
      }

      $query   = new WP_Comment_Query;
      $reviews = $query->query( $args );

      return $reviews;
    }

    //}}
    //{{ _getPageRating

    /**
     * Get reviews list
     *
     * @return int
     */
    protected function _getPageRating( $pageId )
    {
      $total   = 0;
      $rating  = 0;
      $args    = array(
          'type'    => 'iprev_item',
          'status'  => 'approve',
          'post_id' => $pageId
      );
      $query   = new WP_Comment_Query;
      $reviews = $query->query( $args );
      if ( ! empty( $reviews ))
      {
        $count = count( $reviews );
        foreach( $reviews as $review ) {
          $total += $review->comment_karma;
        }
        $per    = $total/$count;
        $round  = round( $per, 1 );
        $roundD = floor( $per );
        $roundU = ceil( $per );
        $roundX = $round*10;

        $mod = $roundX % 5;
        if ( $mod > 5 ) {
          $rating = $roundU;
        } elseif( $mod < 5 && $mod > 0 ) {
          $rating = $roundD + 0.5;
        } else {
          $rating = $roundD;
        }
      }
      return $rating;
    }

    //}}
    //{{ _getStyles

    /**
     * Get styles list
     *
     * @return array
     */
    protected function _getStyles()
    {
      $args = array(
        'orderby'        => 'post_title',
        'order'          => 'DESC',
        'post_type'      => 'iprev-style-item',
        'post_status'    => array('publish'),
        'posts_per_page' => -1,
        'paged'          => 1
      );
      $query   = new WP_Query;
      $videos = $query->query( $args );
      return $videos;
    }

    //}}
    //{{ _getPages

    /**
     * Get review pages list
     *
     * @return array
     */
    protected function _getPages()
    {
      $args = array(
        'orderby'        => 'post_title',
        'order'          => 'DESC',
        'post_type'      => 'iprev-page-item',
        'post_status'    => array('publish'),
        'posts_per_page' => -1,
        'paged'          => 1
      );
      $query   = new WP_Query;
      $videos = $query->query( $args );
      return $videos;
    }

    //}}
    //{{ getShareLink

    /**
     * Get share link
     *
     * @param  string $type
     * @return string
     */
    public function getShareLink( $type )
    {
      $pagetitle = urlencode( get_the_title() );
      $pageurl   = urlencode( get_permalink() );
      $actions   =  array(
        'facebook'      => 'http://www.facebook.com/share.php?u=' . $pageurl,
        'twitter'       => 'https://twitter.com/share?url=' . $pageurl . '&text=' . $pagetitle,
        'google-plus'   => 'https://plusone.google.com/_/+1/confirm?hl=en&url=' . $pageurl,
        'pinterest'     => 'http://pinterest.com/pin/create/button/?url=' . $pageurl . '&media=&description=' . $pagetitle,
        'linkedin'      => 'http://www.linkedin.com/shareArticle?mini=true&url=' . $pageurl .'&title=' . $pagetitle,
        'tumblr'        => 'http://www.tumblr.com/share?v=3&u=' . $pageurl,
        'stumbleupon'   => 'http://www.stumbleupon.com/submit?url=' . $pageurl . '&title=' . $pagetitle,
        'digg'          => 'http://digg.com/submit?url=' . $pageurl . '&title=' . $pagetitle,
        'email'         => 'mailto:?subject=' . $pagetitle
      );
      $link = '#';
      if ( array_key_exists( $type, $actions )) {
        $link = $actions[$type];
      }
      return $link;
    }

    //}}
    //{{ getGoogleFont

    /**
     * Get Google font
     *
     * @return string
     */
    public function getGoogleFont()
    {
        $fonts   = array();
        $fonts[] = $this->setting->getSetting( 'header_font_family', 'roboto' );
        $fonts[] = $this->setting->getSetting( 'footer_font_family', 'roboto' );
        $fonts[] = $this->setting->getSetting( 'main_font_family', 'roboto' );
        $fonts   = array_unique( $fonts );

        $fams  = array();
        $links = IPRev_Font::getLinks();
        foreach( $fonts as $font )
        {
            if ( array_key_exists( $font, $links ))
            {
                $fams[] = $links[$font];
            }
        }

        $family  = implode( '|', $fams );
        $url     = 'http://fonts.googleapis.com/css?family=' . $family;
        return $url;
    }

    //}}
  }
}
