<?php
if ( ! class_exists( 'IPRev_Widget' )) {

    /**
     * Class for handling display widget
     */
    class IPRev_Widget extends WP_Widget
    {
        /**
         * Register widget with WordPress.
         */
        public function __construct()
        {
            parent::__construct(
                'iprev_widget_sidebar', // Base ID
                'Reviews List', // Name
                array( 'description' => __( 'A Widget for display reviews list', IPREV_SLUG ), )
            );
        }

        /**
         * Front-end display of widget.
         *
         * @see WP_Widget::widget()
         *
         * @param array $args     Widget arguments.
         * @param array $instance Saved values from database.
         */
        public function widget( $args, $instance )
        {
            $viewer = new IPRev_Viewer();
            $viewer->widgetFrontend( $instance, $args );
        }

        /**
         * Back-end widget form.
         *
         * @see WP_Widget::form()
         *
         * @param array $instance Previously saved values from database.
         */
        public function form( $instance )
        {
            $viewer = new IPRev_Viewer();
            $viewer->widgetBackend( $instance, $this );
        }

        /**
         * Sanitize widget form values as they are saved.
         *
         * @see WP_Widget::update()
         *
         * @param array $new_instance Values just sent to be saved.
         * @param array $old_instance Previously saved values from database.
         *
         * @return array Updated safe values to be saved.
         */
        public function update( $new_instance, $old_instance )
        {
            $instance = array();
            if ( ! empty( $new_instance['review_page'] ) ) {
                $instance['review_page'] = absint( $new_instance['review_page'] );
            }
            if ( ! empty( $new_instance['style'] ) ) {
                $instance['style'] = absint( $new_instance['style'] );
            }
            return $instance;
        }
    }

}
?>
